/*
 * ******************************************************************************************
 *
 * File		: apb_uart.h
 * Author	: GOWIN
 * Function	: Definition of APB UART
 *
 * ******************************************************************************************
 */

#ifndef __APB_UART_H__
#define __APB_UART_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include <stddef.h>
#include "apb_uart_hal.h"
#include "apb_uart_hw.h"


/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB UART (Universal Asynchronous Receiver/Transmitter)
 ****************************************************************************/
typedef struct
{
	__RO unsigned int IDREV;                /* 0x00 ID and Revision Register           */
	     unsigned int RESERVED0[3];         /* 0x04~0x0C Reserved                      */
	__RO unsigned int CFG;                  /* 0x10 Hardware Configure Register        */
	__RW unsigned int OSCR;                 /* 0x14 Over Sample Control Register       */
	     unsigned int RESERVED1[2];         /* 0x18~0x1C Reserved                      */
	union
	{
		__RO  unsigned int RBR;             /* 0x20 Receiver Buffer Register           */
		__WO  unsigned int THR;             /* 0x20 Transmitter Holding Register       */
		__RW unsigned int DLL;              /* 0x20 Divisor Latch LSB                  */
	};
	union
	{
		__RW unsigned int IER;              /* 0x24 Interrupt Enable Register          */
		__RW unsigned int DLM;              /* 0x24 Divisor Latch MSB                  */
	};
	union
	{
		__RO  unsigned int IIR;             /* 0x28 Interrupt Identification Register  */
		__WO  unsigned int FCR;             /* 0x28 FIFO Control Register              */
	};
	__RW unsigned int LCR;                  /* 0x2C Line Control Register              */
	__RW unsigned int MCR;                  /* 0x30 Modem Control Register             */
	__RW unsigned int LSR;                  /* 0x34 Line Status Register               */
	__RW unsigned int MSR;                  /* 0x38 Modem Status Register              */
	__RW unsigned int SCR;                  /* 0x3C Scratch Register                   */
} APB_UART_RegDef;


/****** APB UART Control Codes *****/

#define APB_UART_CONTROL_Pos                0
#define APB_UART_CONTROL_Msk               (0xFFUL << APB_UART_CONTROL_Pos)

/*----- APB UART Control Codes: Mode -----*/
#define APB_UART_MODE_ASYNCHRONOUS         (0x01UL << APB_UART_CONTROL_Pos)   // APB UART (Asynchronous); arg = Baud rate
#define APB_UART_MODE_SYNCHRONOUS_MASTER   (0x02UL << APB_UART_CONTROL_Pos)   // Synchronous Master (generates clock signal); arg = Baud rate
#define APB_UART_MODE_SYNCHRONOUS_SLAVE    (0x03UL << APB_UART_CONTROL_Pos)   // Synchronous Slave (external clock signal)
#define APB_UART_MODE_SINGLE_WIRE          (0x04UL << APB_UART_CONTROL_Pos)   // APB UART single-wire (half-duplex); arg = Baud rate
#define APB_UART_MODE_IRDA                 (0x05UL << APB_UART_CONTROL_Pos)   // APB UART IrDA; arg = Baud rate
#define APB_UART_MODE_SMART_CARD           (0x06UL << APB_UART_CONTROL_Pos)   // APB UART smart card; arg = Baud rate

/*----- APB UART Control Codes: Mode Parameters: Data Bits -----*/
#define APB_UART_DATA_BITS_Pos              8
#define APB_UART_DATA_BITS_Msk             (7UL << APB_UART_DATA_BITS_Pos)
#define APB_UART_DATA_BITS_5               (5UL << APB_UART_DATA_BITS_Pos)    // 5 data bits
#define APB_UART_DATA_BITS_6               (6UL << APB_UART_DATA_BITS_Pos)    // 6 data bits
#define APB_UART_DATA_BITS_7               (7UL << APB_UART_DATA_BITS_Pos)    // 7 data bits
#define APB_UART_DATA_BITS_8               (0UL << APB_UART_DATA_BITS_Pos)    // 8 data bits (default)
#define APB_UART_DATA_BITS_9               (1UL << APB_UART_DATA_BITS_Pos)    // 9 data bits

/*----- APB UART Control Codes: Mode Parameters: Parity -----*/
#define APB_UART_PARITY_Pos                 12
#define APB_UART_PARITY_Msk                (3UL << APB_UART_PARITY_Pos)
#define APB_UART_PARITY_NONE               (0UL << APB_UART_PARITY_Pos)       // No parity (default)
#define APB_UART_PARITY_EVEN               (1UL << APB_UART_PARITY_Pos)       // Even parity
#define APB_UART_PARITY_ODD                (2UL << APB_UART_PARITY_Pos)       // Odd parity

/*----- APB UART Control Codes: Mode Parameters: Stop Bits -----*/
#define APB_UART_STOP_BITS_Pos              14
#define APB_UART_STOP_BITS_Msk             (3UL << APB_UART_STOP_BITS_Pos)
#define APB_UART_STOP_BITS_1               (0UL << APB_UART_STOP_BITS_Pos)    // 1 stop bits (default)
#define APB_UART_STOP_BITS_2               (1UL << APB_UART_STOP_BITS_Pos)    // 2 stop bits
#define APB_UART_STOP_BITS_1_5             (2UL << APB_UART_STOP_BITS_Pos)    // 1.5 stop bits
#define APB_UART_STOP_BITS_0_5             (3UL << APB_UART_STOP_BITS_Pos)    // 0.5 stop bits

/*----- APB UART Control Codes: Mode Parameters: Flow Control -----*/
#define APB_UART_FLOW_CONTROL_Pos           16
#define APB_UART_FLOW_CONTROL_Msk          (3UL << APB_UART_FLOW_CONTROL_Pos)
#define APB_UART_FLOW_CONTROL_NONE         (0UL << APB_UART_FLOW_CONTROL_Pos) // No flow control (default)
#define APB_UART_FLOW_CONTROL_RTS          (1UL << APB_UART_FLOW_CONTROL_Pos) // RTS flow control
#define APB_UART_FLOW_CONTROL_CTS          (2UL << APB_UART_FLOW_CONTROL_Pos) // CTS flow control
#define APB_UART_FLOW_CONTROL_RTS_CTS      (3UL << APB_UART_FLOW_CONTROL_Pos) // RTS/CTS flow control

/*----- APB UART Control Codes: Mode Parameters: Clock Polarity (Synchronous mode) -----*/
#define APB_UART_CPOL_Pos                   18
#define APB_UART_CPOL_Msk                  (1UL << APB_UART_CPOL_Pos)
#define APB_UART_CPOL0                     (0UL << APB_UART_CPOL_Pos)         // CPOL = 0 (default)
#define APB_UART_CPOL1                     (1UL << APB_UART_CPOL_Pos)         // CPOL = 1

/*----- APB UART Control Codes: Mode Parameters: Clock Phase (Synchronous mode) -----*/
#define APB_UART_CPHA_Pos                   19
#define APB_UART_CPHA_Msk                  (1UL << APB_UART_CPHA_Pos)
#define APB_UART_CPHA0                     (0UL << APB_UART_CPHA_Pos)         // CPHA = 0 (default)
#define APB_UART_CPHA1                     (1UL << APB_UART_CPHA_Pos)         // CPHA = 1

/*----- APB UART Control Codes: Miscellaneous Controls  -----*/
#define APB_UART_SET_DEFAULT_TX_VALUE      (0x10UL << APB_UART_CONTROL_Pos)   // Set default transmit value (Synchronous Receive only); arg = value
#define APB_UART_SET_IRDA_PULSE            (0x11UL << APB_UART_CONTROL_Pos)   // Set IrDA pulse in ns; arg: 0=3/16 of bit period
#define APB_UART_SET_SMART_CARD_GUARD_TIME (0x12UL << APB_UART_CONTROL_Pos)   // Set smart card guard Time; arg = number of bit periods
#define APB_UART_SET_SMART_CARD_CLOCK      (0x13UL << APB_UART_CONTROL_Pos)   // Set smart card clock in Hz; arg: 0=Clock not generated
#define APB_UART_CONTROL_SMART_CARD_NACK   (0x14UL << APB_UART_CONTROL_Pos)   // Smart card NACK generation; arg: 0=disabled, 1=enabled
#define APB_UART_CONTROL_TX                (0x15UL << APB_UART_CONTROL_Pos)   // Transmitter; arg: 0=disabled, 1=enabled
#define APB_UART_CONTROL_RX                (0x16UL << APB_UART_CONTROL_Pos)   // Receiver; arg: 0=disabled, 1=enabled
#define APB_UART_CONTROL_BREAK             (0x17UL << APB_UART_CONTROL_Pos)   // Continuous break transmission; arg: 0=disabled, 1=enabled
#define APB_UART_ABORT_SEND                (0x18UL << APB_UART_CONTROL_Pos)   // Abort APB_UART_Send
#define APB_UART_ABORT_RECEIVE             (0x19UL << APB_UART_CONTROL_Pos)   // Abort APB_UART_Receive
#define APB_UART_ABORT_TRANSFER            (0x1AUL << APB_UART_CONTROL_Pos)   // Abort APB_UART_Transfer


/* General return codes */
#define APB_UART_DRIVER_OK                 0 // Operation succeeded
#define APB_UART_DRIVER_ERROR             -1 // Unspecified error
#define APB_UART_DRIVER_ERROR_BUSY        -2 // Driver is busy
#define APB_UART_DRIVER_ERROR_TIMEOUT     -3 // Timeout occurred
#define APB_UART_DRIVER_ERROR_UNSUPPORTED -4 // Operation not supported
#define APB_UART_DRIVER_ERROR_PARAMETER   -5 // Parameter error
#define APB_UART_DRIVER_ERROR_SPECIFIC    -6 // Start of driver specific errors


/****** APB UART specific error codes *****/
#define APB_UART_ERROR_MODE                (APB_UART_DRIVER_ERROR_SPECIFIC - 1)     // Specified mode not supported
#define APB_UART_ERROR_BAUDRATE            (APB_UART_DRIVER_ERROR_SPECIFIC - 2)     // Specified baud rate not supported
#define APB_UART_ERROR_DATA_BITS           (APB_UART_DRIVER_ERROR_SPECIFIC - 3)     // Specified number of data bits not supported
#define APB_UART_ERROR_PARITY              (APB_UART_DRIVER_ERROR_SPECIFIC - 4)     // Specified parity not supported
#define APB_UART_ERROR_STOP_BITS           (APB_UART_DRIVER_ERROR_SPECIFIC - 5)     // Specified number of stop bits not supported
#define APB_UART_ERROR_FLOW_CONTROL        (APB_UART_DRIVER_ERROR_SPECIFIC - 6)     // Specified flow control not supported
#define APB_UART_ERROR_CPOL                (APB_UART_DRIVER_ERROR_SPECIFIC - 7)     // Specified clock polarity not supported
#define APB_UART_ERROR_CPHA                (APB_UART_DRIVER_ERROR_SPECIFIC - 8)     // Specified clock phase not supported


/***************************************************************************
 * APB UART Status
 ***************************************************************************/
typedef struct _APB_UART_STATUS
{
	unsigned int tx_busy          : 1;        // Transmitter busy flag
	unsigned int rx_busy          : 1;        // Receiver busy flag
	unsigned int tx_underflow     : 1;        // Transmit data underflow detected (cleared on start of next send operation)
	unsigned int rx_overflow      : 1;        // Receive data overflow detected (cleared on start of next receive operation)
	unsigned int rx_break         : 1;        // Break detected on receive (cleared on start of next receive operation)
	unsigned int rx_framing_error : 1;        // Framing error detected on receive (cleared on start of next receive operation)
	unsigned int rx_parity_error  : 1;        // Parity error detected on receive (cleared on start of next receive operation)
} APB_UART_STATUS;

/****************************************************************************
 * APB UART Modem Control
 ****************************************************************************/
typedef enum _APB_UART_MODEM_CONTROL
{
	APB_UART_RTS_CLEAR,                  // Deactivate RTS
	APB_UART_RTS_SET,                    // Activate RTS
	APB_UART_DTR_CLEAR,                  // Deactivate DTR
	APB_UART_DTR_SET                     // Activate DTR
} APB_UART_MODEM_CONTROL;

/****************************************************************************
 * APB UART Modem Status
 ****************************************************************************/
typedef struct _APB_UART_MODEM_STATUS
{
	unsigned int cts : 1;                     // CTS state: 1=Active, 0=Inactive
	unsigned int dsr : 1;                     // DSR state: 1=Active, 0=Inactive
	unsigned int dcd : 1;                     // DCD state: 1=Active, 0=Inactive
	unsigned int ri  : 1;                     // RI  state: 1=Active, 0=Inactive
} APB_UART_MODEM_STATUS;


/****** APB UART Event *****/
#define APB_UART_EVENT_SEND_COMPLETE       (1UL << 0)  // Send completed; however APB UART may still transmit data
#define APB_UART_EVENT_RECEIVE_COMPLETE    (1UL << 1)  // Receive completed
#define APB_UART_EVENT_TRANSFER_COMPLETE   (1UL << 2)  // Transfer completed
#define APB_UART_EVENT_TX_COMPLETE         (1UL << 3)  // Transmit completed (optional)
#define APB_UART_EVENT_TX_UNDERFLOW        (1UL << 4)  // Transmit data not available (Synchronous Slave)
#define APB_UART_EVENT_RX_OVERFLOW         (1UL << 5)  // Receive data overflow
#define APB_UART_EVENT_RX_TIMEOUT          (1UL << 6)  // Receive character timeout (optional)
#define APB_UART_EVENT_RX_BREAK            (1UL << 7)  // Break detected on receive
#define APB_UART_EVENT_RX_FRAMING_ERROR    (1UL << 8)  // Framing error detected on receive
#define APB_UART_EVENT_RX_PARITY_ERROR     (1UL << 9)  // Parity error detected on receive
#define APB_UART_EVENT_CTS                 (1UL << 10) // CTS state changed (optional)
#define APB_UART_EVENT_DSR                 (1UL << 11) // DSR state changed (optional)
#define APB_UART_EVENT_DCD                 (1UL << 12) // DCD state changed (optional)
#define APB_UART_EVENT_RI                  (1UL << 13) // RI  state changed (optional)


/****************************************************************************
 * APB UART Device Driver Capabilities.
 ****************************************************************************/
typedef struct _APB_UART_CAPABILITIES
{
	unsigned int asynchronous       : 1;      // Supports APB UART (Asynchronous) mode
	unsigned int synchronous_master : 1;      // Supports synchronous Master mode
	unsigned int synchronous_slave  : 1;      // Supports synchronous Slave mode
	unsigned int single_wire        : 1;      // Supports APB UART single-wire mode
	unsigned int irda               : 1;      // Supports APB UART IrDA mode
	unsigned int smart_card         : 1;      // Supports APB UART smart card mode
	unsigned int smart_card_clock   : 1;      // Smart card clock generator available
	unsigned int flow_control_rts   : 1;      // RTS flow control available
	unsigned int flow_control_cts   : 1;      // CTS flow control available
	unsigned int event_tx_complete  : 1;      // Transmit completed event: APB_UART_EVENT_TX_COMPLETE
	unsigned int event_rx_timeout   : 1;      // Signal receive character timeout event: APB_UART_EVENT_RX_TIMEOUT
	unsigned int rts                : 1;      // RTS line: 0=not available, 1=available
	unsigned int cts                : 1;      // CTS line: 0=not available, 1=available
	unsigned int dtr                : 1;      // DTR line: 0=not available, 1=available
	unsigned int dsr                : 1;      // DSR line: 0=not available, 1=available
	unsigned int dcd                : 1;      // DCD line: 0=not available, 1=available
	unsigned int ri                 : 1;      // RI line: 0=not available, 1=available
	unsigned int event_cts          : 1;      // Signal CTS change event: APB_UART_EVENT_CTS
	unsigned int event_dsr          : 1;      // Signal DSR change event: APB_UART_EVENT_DSR
	unsigned int event_dcd          : 1;      // Signal DCD change event: APB_UART_EVENT_DCD
	unsigned int event_ri           : 1;      // Signal RI change event: APB_UART_EVENT_RI
} APB_UART_CAPABILITIES;


/* IER Register (+0x24) */
#define APB_UARTC_IER_RDR                   0x01 /* Data ready enable */
#define APB_UARTC_IER_THRE                  0x02 /* THR empty enable */
#define APB_UARTC_IER_RLS                   0x04 /* Receive line status enable */
#define APB_UARTC_IER_MS                    0x08 /* Modem status enable */

/* IIR Register (+0x28) */
#define APB_UARTC_IIR_NONE                  0x01 /* No interrupt pending */
#define APB_UARTC_IIR_RLS                   0x06 /* Receive line status */
#define APB_UARTC_IIR_RDA                   0x04 /* Receive data available */
#define APB_UARTC_IIR_RTO                   0x0c /* Receive time out */
#define APB_UARTC_IIR_THRE                  0x02 /* THR empty */
#define APB_UARTC_IIR_MODEM                 0x00 /* Modem status */
#define APB_UARTC_IIR_INT_MASK              0x0f /* Interrupt status bits mask */

#define APB_UARTC_IIR_TFIFO_FULL            0x10 /* TX FIFO full */
#define APB_UARTC_IIR_FIFO_EN               0xc0 /* FIFO mode is enabled, set when FCR[0] is 1 */

/* FCR Register (+0x28) */
#define APB_UARTC_FCR_FIFO_EN               0x01 /* FIFO enable */
#define APB_UARTC_FCR_RFIFO_RESET           0x02 /* RX FIFO reset */
#define APB_UARTC_FCR_TFIFO_RESET           0x04 /* TT FIFO reset */
#define APB_UARTC_FCR_DMA_EN                0x08 /* Select APB UART DMA mode */

#define APB_UARTC_FCR_TFIFO16_TRGL1         0x00 /* TX 16-byte FIFO interrupt trigger level - 1 char  */
#define APB_UARTC_FCR_TFIFO16_TRGL3         0x10 /* TX 16-byte FIFO interrupt trigger level - 3 char  */
#define APB_UARTC_FCR_TFIFO16_TRGL9         0x20 /* TX 16-byte FIFO interrupt trigger level - 9 char  */
#define APB_UARTC_FCR_TFIFO16_TRGL13        0x30 /* TX 16-byte FIFO interrupt trigger level - 13 char */

#define APB_UARTC_FCR_RFIFO16_TRGL1         0x00 /* RX 16-byte FIFO interrupt trigger level - 1 char  */
#define APB_UARTC_FCR_RFIFO16_TRGL4         0x40 /* RX 16-byte FIFO interrupt trigger level - 4 char  */
#define APB_UARTC_FCR_RFIFO16_TRGL8         0x80 /* RX 16-byte FIFO interrupt trigger level - 8 char  */
#define APB_UARTC_FCR_RFIFO16_TRGL14        0xc0 /* RX 16-byte FIFO interrupt trigger level - 14 char */

#define APB_UARTC_FCR_FIFO_EN_MASK          0x01 /* FIFO enable */
#define APB_UARTC_FCR_FIFO_EN_BIT           0
#define APB_UARTC_FCR_RFIFO_RESET_MASK      0x02 /* RX FIFO reset */
#define APB_UARTC_FCR_RFIFO_RESET_BIT       1
#define APB_UARTC_FCR_TFIFO_RESET_MASK      0x04 /* TX FIFO reset */
#define APB_UARTC_FCR_TFIFO_RESET_BIT       2
#define APB_UARTC_FCR_DMA_EN_MASK           0x08 /* Select APB UART DMA mode */
#define APB_UARTC_FCR_DMA_EN_BIT            3

#define APB_UARTC_FCR_TXFIFO_TRGL_MASK      0x30 /* TX FIFO interrupt trigger level */
#define APB_UARTC_FCR_TXFIFO_TRGL_SHIFT     4
#define APB_UARTC_FCR_RXFIFO_TRGL_MASK      0xc0 /* RX FIFO interrupt trigger level */
#define APB_UARTC_FCR_RXFIFO_TRGL_SHIFT     6

/* LCR Register (+0x2c) */
#define APB_UARTC_LCR_BITS5                 0x00 /* 5 bits */
#define APB_UARTC_LCR_BITS6                 0x01 /* 6 bits */
#define APB_UARTC_LCR_BITS7                 0x02 /* 7 bits */
#define APB_UARTC_LCR_BITS8                 0x03 /* 8 bits */
#define APB_UARTC_LCR_STOP1                 0x00
#define APB_UARTC_LCR_STOP2                 0x04

#define APB_UARTC_LCR_PARITY_EN             0x08 /* Parity enable */
#define APB_UARTC_LCR_PARITY_NONE           0x00 /* No parity check */
#define APB_UARTC_LCR_PARITY_EVEN           0x18 /* Even parity */
#define APB_UARTC_LCR_PARITY_ODD            0x08 /* Odd parity */

#define APB_UARTC_LCR_SETBREAK              0x40 /* Set break condition */
#define APB_UARTC_LCR_DLAB                  0x80 /* Divisor latch access bit */

/* MCR Register (+0x30) */
#define APB_UARTC_MCR_DTR                   0x01 /* Data terminal ready */
#define APB_UARTC_MCR_RTS                   0x02 /* Request to Send */
#define APB_UARTC_MCR_OUT1                  0x04 /* Output1 */
#define APB_UARTC_MCR_OUT2                  0x08 /* Output2 or global interrupt enable */
#define APB_UARTC_MCR_LPBK                  0x10 /* Loop back mode */
#define APB_UARTC_MCR_AFE                   0x20 /* Auto flow control */

/* LSR Register (+0x34) */
#define APB_UARTC_LSR_RDR                   0x1  /* Data ready */
#define APB_UARTC_LSR_OE                    0x2  /* Overrun error */
#define APB_UARTC_LSR_PE                    0x4  /* Parity error */
#define APB_UARTC_LSR_FE                    0x8  /* Framing error */
#define APB_UARTC_LSR_BI                    0x10 /* Break interrupt */
#define APB_UARTC_LSR_THRE                  0x20 /* THR/FIFO empty */
#define APB_UARTC_LSR_TEMT                  0x40 /* THR/FIFO and TFR empty */
#define APB_UARTC_LSR_ERRF                  0x80 /* FIFO data error */

/* MSR Register (+0x38) */
#define APB_UARTC_MSR_DCTS                  0x1  /* Delta CTS */
#define APB_UARTC_MSR_DDSR                  0x2  /* Delta DSR */
#define APB_UARTC_MSR_TERI                  0x4  /* Trailing edge RI */
#define APB_UARTC_MSR_DDCD                  0x8  /* Delta CD */
#define APB_UARTC_MSR_CTS                   0x10 /* Clear to send */
#define APB_UARTC_MSR_DSR                   0x20 /* Data set ready */
#define APB_UARTC_MSR_RI                    0x40 /* Ring indicator */
#define APB_UARTC_MSR_DCD                   0x80 /* Data carrier detect */


// APB UART flags
#define APB_UART_FLAG_INITIALIZED           (1U << 0)
#define APB_UART_FLAG_POWERED               (1U << 1)
#define APB_UART_FLAG_CONFIGURED            (1U << 2)
#define APB_UART_FLAG_TX_ENABLED            (1U << 3)
#define APB_UART_FLAG_RX_ENABLED            (1U << 4)
#define APB_UART_FLAG_SEND_ACTIVE           (1U << 5)

// APB UART TX FIFO trigger level
#define APB_UART_TRIG_LVL_1                 (0x00U)
#define APB_UART_TRIG_LVL_4                 (0x40U)
#define APB_UART_TRIG_LVL_8                 (0x80U)
#define APB_UART_TRIG_LVL_14                (0xC0U)


/****************************************************************************
  \fn          void APB_UART_SignalEvent (unsigned int event)
  \brief       Signal APB UART Events.
  \param[in]   event  \ref APB_UART_events notification mask
  \return      none
*****************************************************************************/
typedef void (*APB_UART_SignalEvent_t) (unsigned int event);  // Pointer to APB_UART_SignalEvent : Signal APB UART Event.


// APB UART transfer information (Run-Time)
typedef struct _APB_UART_TRANSFER_INFO
{
	unsigned int                rx_num;        // Total number of data to be received
	unsigned int                tx_num;        // Total number of data to be send
	unsigned char               *rx_buf;       // Pointer to in data buffer
	unsigned char               *tx_buf;       // Pointer to out data buffer
	unsigned int                rx_cnt;        // Number of data received
	unsigned int                tx_cnt;        // Number of data sent
	unsigned char               tx_def_val;    // Transmit default value (used in APB_UART_SYNC_MASTER_MODE_RX)
	unsigned char               send_active;   // Send active flag
} APB_UART_TRANSFER_INFO;

// APB UART RX status
typedef struct _APB_UART_RX_STATUS
{
	unsigned char rx_busy;                       // Receiver busy flag
	unsigned char rx_overflow;                   // Receive data overflow detected (cleared on start of next receive operation)
	unsigned char rx_break;                      // Break detected on receive (cleared on start of next receive operation)
	unsigned char rx_framing_error;              // Framing error detected on receive (cleared on start of next receive operation)
	unsigned char rx_parity_error;               // Parity error detected on receive (cleared on start of next receive operation)
} APB_UART_RX_STATUS;

// APB UART information (Run-Time)
typedef struct _APB_UART_INFO
{
	APB_UART_SignalEvent_t  cb_event;       // Event callback
	APB_UART_RX_STATUS      rx_status;     	// Receive status flags
	APB_UART_TRANSFER_INFO  xfer;          	// Transfer information
	unsigned char           mode;          	// APB UART mode
	unsigned char           flags;         	// APB UART driver flags
	unsigned int            baudrate;      	// Baud rate
} APB_UART_INFO;

// APB UART DMA
#if (APB_UART_DMA_EN == 1)
typedef const struct _APB_UART_DMA
{
	unsigned char           channel;       	// DMA channel
	unsigned char           reqsel;        	// DMA request selection
	DMA_SignalEvent_t       cb_event;      	// DMA event callback
} APB_UART_DMA;
#endif

// General power states
typedef enum _APB_UART_POWER_STATE
{
	APB_UART_POWER_OFF,                      // Power off: no operation possible
	APB_UART_POWER_LOW,                      // Low power mode: retain state, detect and signal wake-up events
	APB_UART_POWER_FULL                      // Power on: full operation at maximum performance
} APB_UART_POWER_STATE;

// APB UART resources definitions
typedef struct
{
	APB_UART_CAPABILITIES     capabilities;  // Capabilities
	APB_UART_RegDef		      *reg;          // Pointer to APB UART register
	unsigned int              irq_num;       // APB UART IRQ Number
	unsigned int              trig_lvl;      // FIFO trigger level
#if (APB_UART_DMA_EN == 1)
	APB_UART_DMA              *dma_tx;		 // DMA TX
	APB_UART_DMA              *dma_rx;		 // DMA RX
#endif
	APB_UART_INFO             *info;         // Run-Time information
} const APB_UART_RESOURCES;


#ifdef __cplusplus
}
#endif


#endif	/* __APB_UART_H__ */
