/*
 * ******************************************************************************************
 *
 * File		: apb_uart_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB UART
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_uart_driver.h"


// Definitions ------------------------------------------------------------------------------

/****************************************************************************************
  \fn          int apb_uart_set_baudrate (unsigned int         baudrate,
                                          APB_UART_RESOURCES   *apb_uartx)
  \brief       Set baudrate dividers
  \param[in]   baudrate      APB Uart baudrate
  \param[in]   apb_uartx     Pointer to APB UART resources)
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*****************************************************************************************/
static int apb_uart_set_baudrate (unsigned int baudrate, APB_UART_RESOURCES *apb_uartx)
{
	unsigned int div;

	// Over sampling is fixed to 16, and round off the divider value:
	// divider = (_UCLKFREQ / (16.0 * baud rate)) + 0.5;
	// And below is a variant which doesn't use FP operation.
	// It is in RDS_V511
	div = ((_UCLKFREQ + 8 * baudrate) / (16 * baudrate));

	apb_uartx->reg->LCR = 0;
	apb_uartx->reg->LCR = 3;
	apb_uartx->reg->LCR |= APB_UARTC_LCR_DLAB;
	apb_uartx->reg->DLL = (div >> 0) & 0xff;
	apb_uartx->reg->DLM = (div >> 8) & 0xff;
	// Reset DLAB bit
	apb_uartx->reg->LCR &= (~APB_UARTC_LCR_DLAB);

	apb_uartx->info->baudrate = baudrate;

	return 0;
}

/*****************************************************************************************
  \fn          unsigned int apb_uart_rxline_irq_handler (APB_UART_RESOURCES *apb_uartx)
  \brief       Receive line interrupt handler
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      Rx Line event mask
******************************************************************************************/
static unsigned int apb_uart_rxline_irq_handler (APB_UART_RESOURCES *apb_uartx)
{
	unsigned int lsr, event;

	#define APB_UARTC_LSR_LINE_INT      (APB_UARTC_LSR_OE | APB_UARTC_LSR_PE | APB_UARTC_LSR_FE | APB_UARTC_LSR_BI)
	event = 0U;
	lsr   = apb_uartx->reg->LSR  & APB_UARTC_LSR_LINE_INT;

	// OverRun error
	if (lsr & APB_UARTC_LSR_OE)
	{
		apb_uartx->info->rx_status.rx_overflow = 1U;
		event |= APB_UART_EVENT_RX_OVERFLOW;
	}

	// Parity error
	if (lsr & APB_UARTC_LSR_PE)
	{
		apb_uartx->info->rx_status.rx_parity_error = 1U;
		event |= APB_UART_EVENT_RX_PARITY_ERROR;
	}

	// Break detected
	if (lsr & APB_UARTC_LSR_BI)
	{
		apb_uartx->info->rx_status.rx_break = 1U;
		event |= APB_UART_EVENT_RX_BREAK;
	}
	// Framing error
	else
	{
		if (lsr & APB_UARTC_LSR_FE)
		{
			apb_uartx->info->rx_status.rx_framing_error = 1U;
			event |= APB_UART_EVENT_RX_FRAMING_ERROR;
		}
	}

	return event;
}

// APB UART Driver functions

/*****************************************************************************************
  \fn          APB_UART_CAPABILITIES apb_uart_get_capabilities (APB_UART_RESOURCES *apb_uartx)
  \brief       Get driver capabilities
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      \ref APB_UART_CAPABILITIES
******************************************************************************************/
APB_UART_CAPABILITIES apb_uart_get_capabilities (APB_UART_RESOURCES *apb_uartx)
{
	return apb_uartx->capabilities;
}

/*****************************************************************************************
  \fn          int apb_uart_initialize (APB_UART_SignalEvent_t  cb_event
                                        APB_UART_RESOURCES      *apb_uartx)
  \brief       Initialize APB UART Interface.
  \param[in]   cb_event  Pointer to \ref APB_UART_SignalEvent
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
******************************************************************************************/
int apb_uart_initialize (APB_UART_SignalEvent_t cb_event, APB_UART_RESOURCES *apb_uartx)
{
	if (apb_uartx->info->flags & APB_UART_FLAG_INITIALIZED)
	{
		// Driver is already initialized
		return APB_UART_DRIVER_OK;
	}

	// Initialize APB UART Run-time Resources
	apb_uartx->info->cb_event = cb_event;

	apb_uartx->info->rx_status.rx_busy          = 0U;
	apb_uartx->info->rx_status.rx_overflow      = 0U;
	apb_uartx->info->rx_status.rx_break         = 0U;
	apb_uartx->info->rx_status.rx_framing_error = 0U;
	apb_uartx->info->rx_status.rx_parity_error  = 0U;

	apb_uartx->info->xfer.send_active           = 0U;
	apb_uartx->info->xfer.tx_def_val            = 0U;

#if (APB_UART_DMA_EN == 1)
	// DMA Initialize
	if (apb_uartx->dma_tx || apb_uartx->dma_rx)
	{
		dma_initialize ();
	}
#endif

	apb_uartx->info->flags = APB_UART_FLAG_INITIALIZED;

	return APB_UART_DRIVER_OK;
}

/*****************************************************************************************
  \fn          int apb_uart_uninitialize (APB_UART_RESOURCES *apb_uartx)
  \brief       Uninitialized APB UART Interface.
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
******************************************************************************************/
int apb_uart_uninitialize (APB_UART_RESOURCES *apb_uartx)
{
#if (APB_UART_DMA_EN == 1)
	// DMA Un-initialize
	if (apb_uartx->dma_tx || apb_uartx->dma_rx)
	{
		dma_uninitialize ();
	}
#endif

	// Reset APB UART status flags
	apb_uartx->info->flags = 0U;

	return APB_UART_DRIVER_OK;
}

/*****************************************************************************************
  \fn          int apb_uart_power_control (APB_UART_POWER_STATE state,
                                           APB_UART_RESOURCES   *apb_uartx)
  \brief       Control APB UART Interface Power.
  \param[in]   state      Power state
  \param[in]   apb_uartx  Pointer to APB UART resources
  \return      \ref execution_status
******************************************************************************************/
int apb_uart_power_control (APB_UART_POWER_STATE state, APB_UART_RESOURCES *apb_uartx)
{
	unsigned int val;

	switch (state)
	{
	case APB_UART_POWER_OFF:
		// Disable APB UART IRQ
		hal_disable_interrupt(apb_uartx->irq_num);

#if (APB_UART_DMA_EN == 1)
		// If DMA mode - disable TX DMA channel
		if ((apb_uartx->dma_tx) && (apb_uartx->info->xfer.send_active != 0U))
		{
			dma_channel_disable (apb_uartx->dma_tx->channel);
		}

		// If DMA mode - disable DMA channel
		if ((apb_uartx->dma_rx) && (apb_uartx->info->rx_status.rx_busy))
		{
			dma_channel_disable (apb_uartx->dma_rx->channel);
		}
#endif

		// Clear driver variables
		apb_uartx->info->rx_status.rx_busy          = 0U;
		apb_uartx->info->rx_status.rx_overflow      = 0U;
		apb_uartx->info->rx_status.rx_break         = 0U;
		apb_uartx->info->rx_status.rx_framing_error = 0U;
		apb_uartx->info->rx_status.rx_parity_error  = 0U;
		apb_uartx->info->xfer.send_active           = 0U;

		apb_uartx->info->flags &= ~APB_UART_FLAG_POWERED;

		break;

	case APB_UART_POWER_LOW:
		return APB_UART_DRIVER_ERROR_UNSUPPORTED;

	case APB_UART_POWER_FULL:
		if ((apb_uartx->info->flags & APB_UART_FLAG_INITIALIZED) == 0U)
		{
			return APB_UART_DRIVER_ERROR;
		}

		if ((apb_uartx->info->flags & APB_UART_FLAG_POWERED) != 0U)
		{
			return APB_UART_DRIVER_OK;
		}

		// Disable interrupts
		apb_uartx->reg->IER = 0U;

		// Configure FIFO Control register
		// Set trigger level
		val = (apb_uartx->trig_lvl & APB_UARTC_FCR_RXFIFO_TRGL_MASK) | APB_UARTC_FCR_FIFO_EN;

#if (APB_UART_DMA_EN == 1)
		if (apb_uartx->dma_rx || apb_uartx->dma_tx)
		{
			val |= APB_UARTC_FCR_DMA_EN;
		}
#endif

		apb_uartx->reg->FCR = val;

		// Enable modem lines status interrupts
		if (apb_uartx->capabilities.cts || apb_uartx->capabilities.dcd ||
			apb_uartx->capabilities.dsr || apb_uartx->capabilities.ri)
		{
			apb_uartx->reg->IER |= APB_UARTC_IER_MS;
		}

		// Clear driver variables
		apb_uartx->info->rx_status.rx_busy          = 0U;
		apb_uartx->info->rx_status.rx_overflow      = 0U;
		apb_uartx->info->rx_status.rx_break         = 0U;
		apb_uartx->info->rx_status.rx_framing_error = 0U;
		apb_uartx->info->rx_status.rx_parity_error  = 0U;

		apb_uartx->info->mode                       = 0U;
		apb_uartx->info->flags                      = 0U;
		apb_uartx->info->xfer.send_active           = 0U;

		apb_uartx->info->flags = APB_UART_FLAG_POWERED | APB_UART_FLAG_INITIALIZED;

		// Enable APB UART IRQ
		hal_enable_interrupt(apb_uartx->irq_num);

		break;

	default:
		return APB_UART_DRIVER_ERROR_UNSUPPORTED;
	}

	return APB_UART_DRIVER_OK;
}

/******************************************************************************************
  \fn          int apb_uart_send (const void               *data,
                                        unsigned int        num,
                                        APB_UART_RESOURCES *apb_uartx)
  \brief       Start sending data to APB UART transmitter.
  \param[in]   data      Pointer to buffer with data to send to APB UART transmitter
  \param[in]   num       Number of data items to send
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_send (const void *data, unsigned int num, APB_UART_RESOURCES *apb_uartx)
{
#if (APB_UART_DMA_EN == 1)
	int stat;
#endif

	int val;

	if ((data == NULL) || (num == 0U))
	{
		// Invalid parameters
		return APB_UART_DRIVER_ERROR_PARAMETER;
	}

	if ((apb_uartx->info->flags & APB_UART_FLAG_CONFIGURED) == 0U)
	{
		// APB_UART is not configured (mode not selected)
		return APB_UART_DRIVER_ERROR;
	}

	if (apb_uartx->info->xfer.send_active != 0U)
	{
		// Send is not completed yet
		return APB_UART_DRIVER_ERROR_BUSY;
	}

	// Set Send active flag
	apb_uartx->info->xfer.send_active = 1U;

	// Save transmit buffer info
	apb_uartx->info->xfer.tx_buf = (unsigned char *)data;
	apb_uartx->info->xfer.tx_num = num;
	apb_uartx->info->xfer.tx_cnt = 0U;

#if (APB_UART_DMA_EN == 1)
	// DMA mode
	if (apb_uartx->dma_tx)
	{
		// Configure DMA channel
		stat = dma_channel_configure (apb_uartx->dma_tx->channel,
									  (unsigned int)(long)data,
									  (unsigned int)(long)(&(apb_uartx->reg->THR)),
									  num,
									  DMA_CH_CTRL_SBINF(DMA_INF_IDX1) |
									  DMA_CH_CTRL_DBINF(DMA_INF_IDX1) |
									  DMA_CH_CTRL_SBSIZE(DMA_BSIZE_1) |
									  DMA_CH_CTRL_SWIDTH(DMA_WIDTH_BYTE) |
									  DMA_CH_CTRL_DWIDTH(DMA_WIDTH_BYTE) |
									  DMA_CH_CTRL_DMODE_HANDSHAKE |
									  DMA_CH_CTRL_SRCADDR_INC |
									  DMA_CH_CTRL_DSTADDR_FIX |
									  DMA_CH_CTRL_DSTREQ(apb_uartx->dma_tx->reqsel)	|
									  DMA_CH_CTRL_INTABT |
									  DMA_CH_CTRL_INTERR |
									  DMA_CH_CTRL_INTTC	|
									  DMA_CH_CTRL_ENABLE,
									  apb_uartx->dma_tx->cb_event);

		if (stat == -1)
		{
			return APB_UART_DRIVER_ERROR;
		}
	}
	// Interrupt mode
	else
#endif
	{
		// Fill TX FIFO
		if (apb_uartx->reg->LSR & APB_UARTC_LSR_THRE)
		{
			val = 16U;
			while ((val--) && (apb_uartx->info->xfer.tx_cnt != apb_uartx->info->xfer.tx_num))
			{
				apb_uartx->reg->THR = apb_uartx->info->xfer.tx_buf[apb_uartx->info->xfer.tx_cnt++];
			}
		}

		// Enable transmit holding register empty interrupt
		apb_uartx->reg->IER |= APB_UARTC_IER_THRE;
	}

	return APB_UART_DRIVER_OK;
}

/******************************************************************************************
  \fn          int apb_uart_receive (void                *data,
                                     unsigned int         num,
                                     APB_UART_RESOURCES  *apb_uartx)
  \brief       Start receiving data from APB UART receiver.
  \param[out]  data      Pointer to buffer for data to receive from APB UART receiver
  \param[in]   num       Number of data items to receive
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_receive (void *data, unsigned int num, APB_UART_RESOURCES *apb_uartx)
{
#if (APB_UART_DMA_EN == 1)
	int stat;
#endif

	if ((data == NULL) || (num == 0U))
	{
		// Invalid parameters
		return APB_UART_DRIVER_ERROR_PARAMETER;
	}

	if ((apb_uartx->info->flags & APB_UART_FLAG_CONFIGURED) == 0U)
	{
		// APB UART is not configured (mode not selected)
		return APB_UART_DRIVER_ERROR;
	}

	// Check if receiver is busy
	if (apb_uartx->info->rx_status.rx_busy == 1U)
	{
		return APB_UART_DRIVER_ERROR_BUSY;
	}

	// Set RX busy flag
	apb_uartx->info->rx_status.rx_busy = 1U;

	// Save number of data to be received
	apb_uartx->info->xfer.rx_num = num;

	// Clear RX statuses
	apb_uartx->info->rx_status.rx_break          = 0U;
	apb_uartx->info->rx_status.rx_framing_error  = 0U;
	apb_uartx->info->rx_status.rx_overflow       = 0U;
	apb_uartx->info->rx_status.rx_parity_error   = 0U;

	// Save receive buffer info
	apb_uartx->info->xfer.rx_buf = (unsigned char *)data;
	apb_uartx->info->xfer.rx_cnt =              0U;

#if (APB_UART_DMA_EN == 1)
	// DMA mode
	if (apb_uartx->dma_rx)
	{
		stat = dma_channel_configure (apb_uartx->dma_rx->channel,
									  (unsigned int)(long)&apb_uartx->reg->RBR,
									  (unsigned int)(long)data,
									  num,
									  DMA_CH_CTRL_SBINF(DMA_INF_IDX1) |
									  DMA_CH_CTRL_DBINF(DMA_INF_IDX1) |
									  DMA_CH_CTRL_SBSIZE(DMA_BSIZE_1) |
									  DMA_CH_CTRL_SWIDTH(DMA_WIDTH_BYTE) |
									  DMA_CH_CTRL_DWIDTH(DMA_WIDTH_BYTE) |
									  DMA_CH_CTRL_SMODE_HANDSHAKE |
									  DMA_CH_CTRL_SRCADDR_FIX |
									  DMA_CH_CTRL_DSTADDR_INC |
									  DMA_CH_CTRL_SRCREQ(apb_uartx->dma_rx->reqsel)	|
									  DMA_CH_CTRL_INTABT |
									  DMA_CH_CTRL_INTERR |
									  DMA_CH_CTRL_INTTC |
									  DMA_CH_CTRL_ENABLE,
									  apb_uartx->dma_rx->cb_event);

		if (stat == -1)
		{
			return APB_UART_DRIVER_ERROR;
		}
	}
	// Interrupt mode
	else
#endif
	{
		// Enable receive data available interrupt
		apb_uartx->reg->IER |= APB_UARTC_IER_RDR;
	}

	return APB_UART_DRIVER_OK;
}

/******************************************************************************************
  \fn          int apb_uart_transfer (const void               *data_out,
                                            void               *data_in,
                                            unsigned int        num,
                                            APB_UART_RESOURCES *apb_uartx)
  \brief       Start sending/receiving data to/from APB UART transmitter/receiver.
  \param[in]   data_out  Pointer to buffer with data to send to APB UART transmitter
  \param[out]  data_in   Pointer to buffer for data to receive from APB UART receiver
  \param[in]   num       Number of data items to transfer
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_transfer (const void *data_out, void *data_in, unsigned int	num, APB_UART_RESOURCES	*apb_uartx)
{
	if ((data_out == NULL) || (data_in == NULL) || (num == 0U))
	{
		// Invalid parameters
		return APB_UART_DRIVER_ERROR_PARAMETER;
	}

	if ((apb_uartx->info->flags & APB_UART_FLAG_CONFIGURED) == 0U)
	{
		// APB UART is not configured
		return APB_UART_DRIVER_ERROR;
	}

	// Only in synchronous mode
	return APB_UART_DRIVER_ERROR;
}

/******************************************************************************************
  \fn          unsigned int apb_uart_get_txcount (APB_UART_RESOURCES *apb_uartx)
  \brief       Get transmitted data count.
  \param[in]   apb_uartx  Pointer to APB UART resources
  \return      number of data items transmitted
*******************************************************************************************/
unsigned int apb_uart_get_txcount (APB_UART_RESOURCES *apb_uartx)
{
	unsigned int cnt;

#if (APB_UART_DMA_EN == 1)
	if (apb_uartx->dma_tx)
	{
		cnt = dma_channel_get_count (apb_uartx->dma_tx->channel);
	}
	else
#endif
	{
		cnt = apb_uartx->info->xfer.tx_cnt;
	}

	return cnt;
}

/******************************************************************************************
  \fn          unsigned int apb_uart_get_rxcount (APB_UART_RESOURCES *apb_uartx)
  \brief       Get received data count.
  \param[in]   apb_uartx  Pointer to APB UART resources
  \return      number of data items received
*******************************************************************************************/
unsigned int apb_uart_get_rxcount (APB_UART_RESOURCES *apb_uartx)
{
	unsigned int cnt;

#if (APB_UART_DMA_EN == 1)
	if (apb_uartx->dma_rx)
	{
		cnt = dma_channel_get_count (apb_uartx->dma_rx->channel);
	}
	else
#endif
	{
		cnt = apb_uartx->info->xfer.rx_cnt;
	}

	return cnt;
}

/***********************************************************************************************
  \fn          int apb_uart_control (unsigned int          control,
                                     unsigned int          arg,
                                     APB_UART_RESOURCES   *apb_uartx)
  \brief       Control APB UART Interface.
  \param[in]   control   Operation
  \param[in]   arg       Argument of operation (optional)
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      common \ref execution_status and driver specific \ref apb_uart_execution_status
************************************************************************************************/
int apb_uart_control (unsigned int control, unsigned int arg, APB_UART_RESOURCES  *apb_uartx)
{
	unsigned int val, mode;
	unsigned int lcr, mcr;

	if ((apb_uartx->info->flags & APB_UART_FLAG_POWERED) == 0U)
	{
		// APB UART not powered
		return APB_UART_DRIVER_ERROR;
	}

	lcr      = 0U;
	mcr      = 0U;

	switch (control & APB_UART_CONTROL_Msk)
	{
	// Control TX
	case APB_UART_CONTROL_TX:
		// Check if TX line available
		if (arg)
		{
			apb_uartx->info->flags |= APB_UART_FLAG_TX_ENABLED;
		}
		else
		{
			apb_uartx->info->flags &= ~APB_UART_FLAG_TX_ENABLED;
		}

		return APB_UART_DRIVER_OK;

	// Control RX
	case APB_UART_CONTROL_RX:
		// RX Line interrupt enable (overrun, framing, parity error, break)
		if (arg)
		{
			apb_uartx->info->flags |= APB_UART_FLAG_RX_ENABLED;
			apb_uartx->reg->IER |= APB_UARTC_IER_RLS;
		}
		else
		{
			apb_uartx->info->flags &= ~APB_UART_FLAG_RX_ENABLED;
			apb_uartx->reg->IER &= ~APB_UARTC_IER_RLS;
		}

		return APB_UART_DRIVER_OK;

	// Control break
	case APB_UART_CONTROL_BREAK:
		if (arg)
		{
			if (apb_uartx->info->xfer.send_active != 0U)
			{
				return APB_UART_DRIVER_ERROR_BUSY;
			}

			apb_uartx->reg->LCR |= APB_UARTC_LCR_SETBREAK;

			// Set Send active flag
			apb_uartx->info->xfer.send_active = 1U;
		}
		else
		{
			apb_uartx->reg->LCR &= ~APB_UARTC_LCR_SETBREAK;

			// Clear Send active flag
			apb_uartx->info->xfer.send_active = 0U;
		}

		return APB_UART_DRIVER_OK;

	// Abort Send
	case APB_UART_ABORT_SEND:
		// Disable transmit holding register empty interrupt
		apb_uartx->reg->IER &= ~APB_UARTC_IER_THRE;

		// Set trigger level
		val  = (apb_uartx->trig_lvl & APB_UARTC_FCR_RXFIFO_TRGL_MASK) |	APB_UARTC_FCR_FIFO_EN;
#if (APB_UART_DMA_EN == 1)
		if (apb_uartx->dma_rx || apb_uartx->dma_tx)
		{
			val |= APB_UARTC_FCR_DMA_EN;
		}
#endif

		// Transmit FIFO reset
		val |= APB_UARTC_FCR_TFIFO_RESET;
		apb_uartx->reg->FCR = val;

#if (APB_UART_DMA_EN == 1)
		// If DMA mode - disable DMA channel
		if ((apb_uartx->dma_tx) && (apb_uartx->info->xfer.send_active != 0U))
		{
			dma_channel_disable (apb_uartx->dma_tx->channel);
		}
#endif

		// Clear Send active flag
		apb_uartx->info->xfer.send_active = 0U;

		return APB_UART_DRIVER_OK;

	// Abort receive
	case APB_UART_ABORT_RECEIVE:
		// Disable receive data available interrupt
		apb_uartx->reg->IER &= ~APB_UARTC_IER_RDR;

		// Set trigger level
		val  = (apb_uartx->trig_lvl & APB_UARTC_FCR_RXFIFO_TRGL_MASK) |	APB_UARTC_FCR_FIFO_EN;
#if (APB_UART_DMA_EN == 1)
		if (apb_uartx->dma_rx || apb_uartx->dma_tx)
		{
			val |= APB_UARTC_FCR_DMA_EN;
		}
#endif

		// Receive FIFO reset
		val |= APB_UARTC_FCR_RFIFO_RESET;
		apb_uartx->reg->FCR = val;

#if (APB_UART_DMA_EN == 1)
		// If DMA mode - disable DMA channel
		if ((apb_uartx->dma_rx) && (apb_uartx->info->rx_status.rx_busy))
		{
			dma_channel_disable (apb_uartx->dma_rx->channel);
		}
#endif

		// Clear RX busy status
		apb_uartx->info->rx_status.rx_busy = 0U;

		return APB_UART_DRIVER_OK;

	// Abort transfer
	case APB_UART_ABORT_TRANSFER:
		// Disable transmit holding register empty and receive data available interrupts
		apb_uartx->reg->IER &= ~(APB_UARTC_IER_THRE | APB_UARTC_IER_RDR);

#if (APB_UART_DMA_EN == 1)
		// If DMA mode - disable DMA channel
		if ((apb_uartx->dma_tx) && (apb_uartx->info->xfer.send_active != 0U))
		{
			dma_channel_disable (apb_uartx->dma_tx->channel);
		}

		if ((apb_uartx->dma_rx) && (apb_uartx->info->rx_status.rx_busy))
		{
			dma_channel_disable (apb_uartx->dma_rx->channel);
		}
#endif

		// Set trigger level
		val  = (apb_uartx->trig_lvl & APB_UARTC_FCR_RXFIFO_TRGL_MASK) | APB_UARTC_FCR_FIFO_EN;
#if (APB_UART_DMA_EN == 1)
		if (apb_uartx->dma_rx || apb_uartx->dma_tx)
		{
			val |= APB_UARTC_FCR_DMA_EN;
		}
#endif

		// Transmit and receive FIFO reset
		val |= APB_UARTC_FCR_TFIFO_RESET | APB_UARTC_FCR_RFIFO_RESET;
		apb_uartx->reg->FCR = val;

		// Clear busy statuses
		apb_uartx->info->rx_status.rx_busy = 0U;
		apb_uartx->info->xfer.send_active  = 0U;

		return APB_UART_DRIVER_OK;

	default:
		break;
	}

	switch (control & APB_UART_CONTROL_Msk)
	{
	case APB_UART_MODE_ASYNCHRONOUS:
		mode = APB_UART_MODE_ASYNCHRONOUS;
		break;

	case APB_UART_MODE_SYNCHRONOUS_MASTER:
	case APB_UART_MODE_SYNCHRONOUS_SLAVE:
	case APB_UART_MODE_IRDA:
	case APB_UART_MODE_SMART_CARD:
	case APB_UART_MODE_SINGLE_WIRE:
		return APB_UART_ERROR_MODE;

	// Default TX value
	case APB_UART_SET_DEFAULT_TX_VALUE:
		apb_uartx->info->xfer.tx_def_val = arg;
		return APB_UART_DRIVER_OK;

	// Unsupported command
	default:
		return APB_UART_DRIVER_ERROR_UNSUPPORTED;
	}

	// Check if Receiver/Transmitter is busy
	if (apb_uartx->info->rx_status.rx_busy ||
	   (apb_uartx->info->xfer.send_active != 0U))
	{
		return APB_UART_DRIVER_ERROR_BUSY;
	}

	// APB UART data bits
	switch (control & APB_UART_DATA_BITS_Msk)
	{
	case APB_UART_DATA_BITS_5:
		lcr |= APB_UARTC_LCR_BITS5;
		break;
	case APB_UART_DATA_BITS_6:
		lcr |= APB_UARTC_LCR_BITS6;
		break;
	case APB_UART_DATA_BITS_7:
		lcr |= APB_UARTC_LCR_BITS7;
		break;
	case APB_UART_DATA_BITS_8:
		lcr |= APB_UARTC_LCR_BITS8;
		break;
	default:
		return APB_UART_ERROR_DATA_BITS;
	}

	// APB UART parity
	switch (control & APB_UART_PARITY_Msk)
	{
	case APB_UART_PARITY_NONE:
		break;
	case APB_UART_PARITY_EVEN:
		lcr |= APB_UARTC_LCR_PARITY_EVEN;
		break;
	case APB_UART_PARITY_ODD:
		lcr |= APB_UARTC_LCR_PARITY_ODD;
		break;
	default:
		return (APB_UART_ERROR_PARITY);
	}

	// APB UART stop bits
	switch (control & APB_UART_STOP_BITS_Msk)
	{
	case APB_UART_STOP_BITS_1:
		break;
	case APB_UART_STOP_BITS_2:
		lcr |= APB_UARTC_LCR_STOP2;
		break;
	default:
		return APB_UART_ERROR_STOP_BITS;
	}

	mcr = apb_uartx->reg->MCR & ~APB_UARTC_MCR_AFE;

	// APB UART flow control
	switch (control & APB_UART_FLOW_CONTROL_Msk)
	{
	case APB_UART_FLOW_CONTROL_NONE:
		break;
	case APB_UART_FLOW_CONTROL_RTS:
		if (apb_uartx->capabilities.flow_control_rts)
		{
			mcr |= APB_UARTC_MCR_AFE;
		}
		else
		{
			return APB_UART_ERROR_FLOW_CONTROL;
		}
		break;
	case APB_UART_FLOW_CONTROL_CTS:
		if (apb_uartx->capabilities.flow_control_cts)
		{
			mcr |= APB_UARTC_MCR_AFE;
		}
		else
		{
			return APB_UART_ERROR_FLOW_CONTROL;
		}
		break;
	case APB_UART_FLOW_CONTROL_RTS_CTS:
		if (apb_uartx->capabilities.flow_control_rts &&
			apb_uartx->capabilities.flow_control_cts)
		{
			mcr |= APB_UARTC_MCR_AFE;
		}
		else
		{
			return APB_UART_ERROR_FLOW_CONTROL;
		}
		break;
	default:
		return APB_UART_ERROR_FLOW_CONTROL;
	}

	// APB UART baud rate
	if (apb_uart_set_baudrate (arg, apb_uartx) == -1)
	{
		return APB_UART_ERROR_BAUDRATE;
	}

	// Configuration is OK - Mode is valid
	apb_uartx->info->mode = mode;

	// Configure MCR register
	apb_uartx->reg->MCR = (apb_uartx->reg->MCR & ~APB_UARTC_MCR_AFE) | mcr;

	// Configure Line control register
	apb_uartx->reg->LCR = ((apb_uartx->reg->LCR & (APB_UARTC_LCR_SETBREAK | APB_UARTC_LCR_DLAB)) | lcr);

	// Set configured flag
	apb_uartx->info->flags |= APB_UART_FLAG_CONFIGURED;

	return APB_UART_DRIVER_OK;
}

/******************************************************************************************
  \fn          APB_UART_STATUS apb_uart_get_status (APB_UART_RESOURCES *apb_uartx)
  \brief       Get APB UART status.
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      APB_UART status \ref APB_UART_STATUS
*******************************************************************************************/
APB_UART_STATUS apb_uart_get_status (APB_UART_RESOURCES *apb_uartx)
{
	APB_UART_STATUS stat;

	stat.tx_busy          = (apb_uartx->reg->LSR & APB_UARTC_LSR_TEMT ? (0U) : (1U));
	stat.rx_busy          = apb_uartx->info->rx_status.rx_busy;
	stat.tx_underflow     = 0U;
	stat.rx_overflow      = apb_uartx->info->rx_status.rx_overflow;
	stat.rx_break         = apb_uartx->info->rx_status.rx_break;
	stat.rx_framing_error = apb_uartx->info->rx_status.rx_framing_error;
	stat.rx_parity_error  = apb_uartx->info->rx_status.rx_parity_error;

	return stat;
}

/******************************************************************************************
  \fn          int apb_uart_set_modem_control (APB_UART_MODEM_CONTROL  control,
                                               APB_UART_RESOURCES      *apb_uartx)
  \brief       Set APB UART Modem Control line state.
  \param[in]   control   \ref APB_UART_MODEM_CONTROL
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_set_modem_control (APB_UART_MODEM_CONTROL control, APB_UART_RESOURCES *apb_uartx)
{
	if ((apb_uartx->info->flags & APB_UART_FLAG_CONFIGURED) == 0U)
	{
		// APB UART is not configured
		return APB_UART_DRIVER_ERROR;
	}

	if (control == APB_UART_RTS_CLEAR)
	{
		if (apb_uartx->capabilities.rts)
		{
			apb_uartx->reg->MCR &= ~APB_UARTC_MCR_RTS;
		}
		else
		{
			return APB_UART_DRIVER_ERROR_UNSUPPORTED;
		}
	}

	if (control == APB_UART_RTS_SET)
	{
		if (apb_uartx->capabilities.rts)
		{
			apb_uartx->reg->MCR |=  APB_UARTC_MCR_RTS;
		}
		else
		{
			return APB_UART_DRIVER_ERROR_UNSUPPORTED;
		}
	}

	if (control == APB_UART_DTR_CLEAR)
	{
		if (apb_uartx->capabilities.dtr)
		{
			apb_uartx->reg->MCR &= ~APB_UARTC_MCR_DTR;
		}
		else
		{
			return APB_UART_DRIVER_ERROR_UNSUPPORTED;
		}
	}

	if (control == APB_UART_DTR_SET)
	{
		if (apb_uartx->capabilities.dtr)
		{
			apb_uartx->reg->MCR |=  APB_UARTC_MCR_DTR;
		}
		else
		{
			return APB_UART_DRIVER_ERROR_UNSUPPORTED;
		}
	}

	return APB_UART_DRIVER_OK;
}

/******************************************************************************************
  \fn          APB_UART_MODEM_STATUS apb_uart_get_modem_status (APB_UART_RESOURCES *apb_uartx)
  \brief       Get APB UART Modem Status lines state.
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      modem status \ref APB_UART_MODEM_STATUS
*******************************************************************************************/
APB_UART_MODEM_STATUS apb_uart_get_modem_status (APB_UART_RESOURCES *apb_uartx)
{
	APB_UART_MODEM_STATUS modem_status;
	unsigned int msr;

	if (apb_uartx->info->flags & APB_UART_FLAG_CONFIGURED)
	{
		msr = apb_uartx->reg->MSR;

		if (apb_uartx->capabilities.cts)
		{
			modem_status.cts = (msr & APB_UARTC_MSR_CTS ? (1U) : (0U));
		}
		else
		{
			modem_status.cts = 0U;
		}

		if (apb_uartx->capabilities.dsr)
		{
			modem_status.dsr = (msr & APB_UARTC_MSR_DSR ? (1U) : (0U));
		}
		else
		{
			modem_status.dsr = 0U;
		}

		if (apb_uartx->capabilities.ri )
		{
			modem_status.ri  = (msr & APB_UARTC_MSR_RI  ? (1U) : (0U));
		}
		else
		{
			modem_status.ri  = 0U;
		}

		if (apb_uartx->capabilities.dcd)
		{
			modem_status.dcd = (msr & APB_UARTC_MSR_DCD ? (1U) : (0U));
		}
		else
		{
			modem_status.dcd = 0U;
		}

	}
	else
	{
		modem_status.cts = 0U;
		modem_status.dsr = 0U;
		modem_status.ri  = 0U;
		modem_status.dcd = 0U;
	}

	return modem_status;
}

/******************************************************************************************
  \fn          void apb_uart_irq_handler (APB_UART_RESOURCES *apb_uartx)
  \brief       APB UART Interrupt handler.
  \param[in]   apb_uartx     Pointer to APB UART resources
*******************************************************************************************/
void apb_uart_irq_handler (APB_UART_RESOURCES *apb_uartx)
{
	unsigned int iir, event, val, i;

	event = 0U;
	iir   = apb_uartx->reg->IIR;

	if ((iir & APB_UARTC_IIR_NONE) == 0U)
	{
		// Transmit holding register empty
		if ((iir & APB_UARTC_IIR_INT_MASK) == APB_UARTC_IIR_THRE)
		{
			val = 16U;
			while ((val --) && (apb_uartx->info->xfer.tx_num != apb_uartx->info->xfer.tx_cnt))
			{
				// Write data to TX FIFO
				apb_uartx->reg->THR = apb_uartx->info->xfer.tx_buf[apb_uartx->info->xfer.tx_cnt];
				apb_uartx->info->xfer.tx_cnt++;
			}

			// Check if all data is transmitted
			if (apb_uartx->info->xfer.tx_num == apb_uartx->info->xfer.tx_cnt)
			{
				// Disable THRE interrupt
				apb_uartx->reg->IER &= ~APB_UARTC_IER_THRE;

				// Clear TX busy flag
				apb_uartx->info->xfer.send_active = 0U;

				// Set send complete event
				event |= APB_UART_EVENT_SEND_COMPLETE;
			}
		}

		// Receive line status
		if ((iir & APB_UARTC_IIR_INT_MASK) == APB_UARTC_IIR_RLS)
		{
			event |= apb_uart_rxline_irq_handler(apb_uartx);
		}

		// Receive data available and Character time-out indicator interrupt
		if (((iir & APB_UARTC_IIR_INT_MASK) == APB_UARTC_IIR_RDA)  ||
			((iir & APB_UARTC_IIR_INT_MASK) == APB_UARTC_IIR_RTO))
		{
			switch (apb_uartx->trig_lvl)
			{
			case APB_UART_TRIG_LVL_1:
				i = 1U;
				break;
			case APB_UART_TRIG_LVL_4:
				i = 3U;
				break;
			case APB_UART_TRIG_LVL_8:
				i = 7U;
				break;
			case APB_UART_TRIG_LVL_14:
				i = 13U;
				break;
			default:
				i = 1U;
				break;
			}

			// Get available data from RX FIFO
			while ((apb_uartx->reg->LSR & APB_UARTC_LSR_RDR) && (i--))
			{
				// Check RX line interrupt for errors
				event |= apb_uart_rxline_irq_handler(apb_uartx);

				// Read data from RX FIFO into receive buffer
				apb_uartx->info->xfer.rx_buf[apb_uartx->info->xfer.rx_cnt] = apb_uartx->reg->RBR;

				apb_uartx->info->xfer.rx_cnt++;

				// Check if requested amount of data is received
				if (apb_uartx->info->xfer.rx_cnt == apb_uartx->info->xfer.rx_num)
				{
					// Disable RDA interrupt
					apb_uartx->reg->IER &= ~APB_UARTC_IER_RDR;

					// Clear RX busy flag and set receive transfer complete event
					apb_uartx->info->rx_status.rx_busy = 0U;
					event |= APB_UART_EVENT_RECEIVE_COMPLETE;

					break;
				}
			}
		}

		// Character time-out indicator
		if ((iir & APB_UARTC_IIR_INT_MASK) == APB_UARTC_IIR_RTO)
		{
			// Signal RX Time-out event, if not all requested data received
			if (apb_uartx->info->xfer.rx_cnt != apb_uartx->info->xfer.rx_num)
			{
				event |= APB_UART_EVENT_RX_TIMEOUT;
			}
		}

		// Modem interrupt
		if ((iir & APB_UARTC_IIR_INT_MASK) == APB_UARTC_IIR_MODEM)
		{
			// Save modem status register
			val = apb_uartx->reg->MSR;

			// CTS state changed
			if ((apb_uartx->capabilities.cts) && (val & APB_UARTC_MSR_DCTS))
			{
				event |= APB_UART_EVENT_CTS;
			}

			// DSR state changed
			if ((apb_uartx->capabilities.dsr) && (val & APB_UARTC_MSR_DDSR))
			{
				event |= APB_UART_EVENT_DSR;
			}

			// Ring indicator
			if ((apb_uartx->capabilities.ri)  && (val & APB_UARTC_MSR_TERI))
			{
				event |= APB_UART_EVENT_RI;
			}

			// DCD state changed
			if ((apb_uartx->capabilities.dcd) && (val & APB_UARTC_MSR_DDCD))
			{
				event |= APB_UART_EVENT_DCD;
			}

		}
	}

	if ((apb_uartx->info->cb_event != NULL) && (event != 0U))
	{
		apb_uartx->info->cb_event (event);
	}
}

#if (APB_UART_DMA_EN == 1)
/*********************************************************************************************
  \fn          void apb_uart_dma_tx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx)
  \brief       APB UART DMA TX event.
  \param[in]   apb_uartx Pointer to APB UART resources
  \param[in]   event     DMA_EVENT_TERMINAL_COUNT_REQUEST / DMA_EVENT_ERROR
**********************************************************************************************/
void apb_uart_dma_tx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx)
{
	switch (event)
	{
	case DMA_EVENT_TERMINAL_COUNT_REQUEST:
		apb_uartx->info->xfer.tx_cnt = apb_uartx->info->xfer.tx_num;
		// Clear TX busy flag
		apb_uartx->info->xfer.send_active = 0U;

		// Set send complete event for asynchronous transfers
		if (apb_uartx->info->cb_event)
		{
			apb_uartx->info->cb_event (APB_UART_EVENT_SEND_COMPLETE);
		}

		break;

	case DMA_EVENT_ERROR:
		break;
	default:
		break;
	}
}

/*********************************************************************************************
  \fn          void apb_uart_dma_rx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx)
  \brief       APB UART DMA RX event.
  \param[in]   event     DMA_EVENT_TERMINAL_COUNT_REQUEST / DMA_EVENT_ERROR
  \param[in]   apb_uartx Pointer to APB UART resources
**********************************************************************************************/
void apb_uart_dma_rx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx)
{
	switch (event)
	{
	case DMA_EVENT_TERMINAL_COUNT_REQUEST:
		apb_uartx->info->xfer.rx_cnt    = apb_uartx->info->xfer.rx_num;
		apb_uartx->info->rx_status.rx_busy = 0U;

		if (apb_uartx->info->cb_event)
		{
			apb_uartx->info->cb_event (APB_UART_EVENT_RECEIVE_COMPLETE);
		}

		break;

	case DMA_EVENT_ERROR:
		break;
	default:
		break;
	}
}
#endif
