/*
 * ******************************************************************************************
 *
 * File		: apb_uart_driver.h
 * Author	: GOWIN
 * Function	: Driver definition of APB UART
 *
 * ******************************************************************************************
 */

#ifndef __APB_UART_DRIVER_H__
#define __APB_UART_DRIVER_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_uart.h"


// Definitions ------------------------------------------------------------------------------

/*****************************************************************************************
  \fn          APB_UART_CAPABILITIES apb_uart_get_capabilities (APB_UART_RESOURCES *apb_uartx)
  \brief       Get driver capabilities
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      \ref APB_UART_CAPABILITIES
******************************************************************************************/
APB_UART_CAPABILITIES apb_uart_get_capabilities (APB_UART_RESOURCES *apb_uartx);

/*****************************************************************************************
  \fn          int apb_uart_initialize (APB_UART_SignalEvent_t  cb_event
                                        APB_UART_RESOURCES      *apb_uartx)
  \brief       Initialize APB UART Interface.
  \param[in]   cb_event  Pointer to \ref APB_UART_SignalEvent
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
******************************************************************************************/
int apb_uart_initialize (APB_UART_SignalEvent_t cb_event, APB_UART_RESOURCES *apb_uartx);

/*****************************************************************************************
  \fn          int apb_uart_uninitialize (APB_UART_RESOURCES *apb_uartx)
  \brief       Uninitialized APB UART Interface.
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
******************************************************************************************/
int apb_uart_uninitialize (APB_UART_RESOURCES *apb_uartx);

/*****************************************************************************************
  \fn          int apb_uart_power_control (APB_UART_POWER_STATE state,
                                           APB_UART_RESOURCES   *apb_uartx)
  \brief       Control APB UART Interface Power.
  \param[in]   state      Power state
  \param[in]   apb_uartx  Pointer to APB UART resources
  \return      \ref execution_status
******************************************************************************************/
int apb_uart_power_control (APB_UART_POWER_STATE state, APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          int apb_uart_send (const void               *data,
                                        unsigned int        num,
                                        APB_UART_RESOURCES *apb_uartx)
  \brief       Start sending data to APB UART transmitter.
  \param[in]   data      Pointer to buffer with data to send to APB UART transmitter
  \param[in]   num       Number of data items to send
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_send (const void *data, unsigned int num, APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          int apb_uart_receive (void                *data,
                                     unsigned int         num,
                                     APB_UART_RESOURCES  *apb_uartx)
  \brief       Start receiving data from APB UART receiver.
  \param[out]  data      Pointer to buffer for data to receive from APB UART receiver
  \param[in]   num       Number of data items to receive
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_receive (void *data, unsigned int num, APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          int apb_uart_transfer (const void               *data_out,
                                            void               *data_in,
                                            unsigned int        num,
                                            APB_UART_RESOURCES *apb_uartx)
  \brief       Start sending/receiving data to/from APB UART transmitter/receiver.
  \param[in]   data_out  Pointer to buffer with data to send to APB UART transmitter
  \param[out]  data_in   Pointer to buffer for data to receive from APB UART receiver
  \param[in]   num       Number of data items to transfer
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_transfer (const void *data_out, void *data_in, unsigned int	num, APB_UART_RESOURCES	*apb_uartx);

/******************************************************************************************
  \fn          unsigned int apb_uart_get_txcount (APB_UART_RESOURCES *apb_uartx)
  \brief       Get transmitted data count.
  \param[in]   apb_uartx  Pointer to APB UART resources
  \return      number of data items transmitted
*******************************************************************************************/
unsigned int apb_uart_get_txcount (APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          unsigned int apb_uart_get_rxcount (APB_UART_RESOURCES *apb_uartx)
  \brief       Get received data count.
  \param[in]   apb_uartx  Pointer to APB UART resources
  \return      number of data items received
*******************************************************************************************/
unsigned int apb_uart_get_rxcount (APB_UART_RESOURCES *apb_uartx);

/***********************************************************************************************
  \fn          int apb_uart_control (unsigned int          control,
                                     unsigned int          arg,
                                     APB_UART_RESOURCES   *apb_uartx)
  \brief       Control APB UART Interface.
  \param[in]   control   Operation
  \param[in]   arg       Argument of operation (optional)
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      common \ref execution_status and driver specific \ref apb_uart_execution_status
************************************************************************************************/
int apb_uart_control (unsigned int control, unsigned int arg, APB_UART_RESOURCES  *apb_uartx);

/******************************************************************************************
  \fn          APB_UART_STATUS apb_uart_get_status (APB_UART_RESOURCES *apb_uartx)
  \brief       Get APB UART status.
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      APB_UART status \ref APB_UART_STATUS
*******************************************************************************************/
APB_UART_STATUS apb_uart_get_status (APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          int apb_uart_set_modem_control (APB_UART_MODEM_CONTROL  control,
                                               APB_UART_RESOURCES      *apb_uartx)
  \brief       Set APB UART Modem Control line state.
  \param[in]   control   \ref APB_UART_MODEM_CONTROL
  \param[in]   apb_uartx Pointer to APB UART resources
  \return      \ref execution_status
*******************************************************************************************/
int apb_uart_set_modem_control (APB_UART_MODEM_CONTROL control, APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          APB_UART_MODEM_STATUS apb_uart_get_modem_status (APB_UART_RESOURCES *apb_uartx)
  \brief       Get APB UART Modem Status lines state.
  \param[in]   apb_uartx     Pointer to APB UART resources
  \return      modem status \ref APB_UART_MODEM_STATUS
*******************************************************************************************/
APB_UART_MODEM_STATUS apb_uart_get_modem_status (APB_UART_RESOURCES *apb_uartx);

/******************************************************************************************
  \fn          void apb_uart_irq_handler (APB_UART_RESOURCES *apb_uartx)
  \brief       APB UART Interrupt handler.
  \param[in]   apb_uartx     Pointer to APB UART resources
*******************************************************************************************/
void apb_uart_irq_handler (APB_UART_RESOURCES *apb_uartx);

#if (APB_UART_DMA_EN == 1)
/*********************************************************************************************
  \fn          void apb_uart_dma_tx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx)
  \brief       APB UART DMA TX event.
  \param[in]   apb_uartx Pointer to APB UART resources
  \param[in]   event     DMA_EVENT_TERMINAL_COUNT_REQUEST / DMA_EVENT_ERROR
**********************************************************************************************/
void apb_uart_dma_tx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx);

/*********************************************************************************************
  \fn          void apb_uart_dma_rx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx)
  \brief       APB UART DMA RX event.
  \param[in]   event     DMA_EVENT_TERMINAL_COUNT_REQUEST / DMA_EVENT_ERROR
  \param[in]   apb_uartx Pointer to APB UART resources
**********************************************************************************************/
void apb_uart_dma_rx_event (unsigned int event, APB_UART_RESOURCES *apb_uartx);
#endif


#ifdef __cplusplus
}
#endif


#endif	/* __APB_UART_DRIVER_H__ */
