/*
 * ******************************************************************************************
 *
 * File		: main.c
 * Author	: GOWIN
 * Function	: APB UART demo
 *
 * ******************************************************************************************
 */

/*
 ********************************************************************************************
 * This example program shows the usage of the APB UART interface for asynchronous
 * communication. It demonstrates the APB UART to send/receive to/from a PC via
 * a terminal emulation program (HyperTerm, TeraTerm).
 *
 * Scenario:
 *
 * After initialized, the APB UART will send out the string "Input 'G' to receive
 * a message" and then wait for receiving data. If the "carriage return" is received
 * (i.e. the 'G' key is pressed on PC side under terminal emulation program),
 * the APB UART will send out the "Hello World!" welcome message. Otherwise, it will
 * bypass the received data and continue to wait for next receiving.
 ********************************************************************************************
 */

// Includes ---------------------------------------------------------------------------------
#include "apb_uart_dev.h"


// Definitions ------------------------------------------------------------------------------

static volatile char uart_event_complete = 0;

static void wait_complete(void)
{
	while (!uart_event_complete);
	uart_event_complete = 0;
}

// APB UART1 callback event
void myUART_callback(unsigned int event)
{
    switch (event)
    {
    	case APB_UART_EVENT_RECEIVE_COMPLETE:
    	case APB_UART_EVENT_TRANSFER_COMPLETE:
    	case APB_UART_EVENT_SEND_COMPLETE:
    	case APB_UART_EVENT_TX_COMPLETE:
    		/* Success: continue task */
    		uart_event_complete = 1;
    		break;

    	case APB_UART_EVENT_RX_TIMEOUT:
    		while(1);
    		break;

    	case APB_UART_EVENT_RX_OVERFLOW:
    	case APB_UART_EVENT_TX_UNDERFLOW:
    		while(1);
    		break;
    	default:
    		while(1);
    		break;
    }
}

// Application entry function
int main(void)
{
    char cmd = 0;

    /* Initializes the APB UART1 driver */
    apb_uart1_initialize(myUART_callback);

    /* Power up the APB UART1 peripheral */
    apb_uart1_power_control(APB_UART_POWER_FULL);

    /* Configure the APB UART1 to 38400 Bits/second */
    apb_uart1_control(APB_UART_MODE_ASYNCHRONOUS |			// Mode			: asynchronous
    				  APB_UART_DATA_BITS_8 |				// Data bits	: 8-bit
					  APB_UART_PARITY_NONE |				// Parity		: none
					  APB_UART_STOP_BITS_1 |				// Stop bit:    : 1-bit
					  APB_UART_FLOW_CONTROL_NONE, 38400);	// Flow control : none
    														// Baud rate    : 38400

    /* Enable Receiver and Transmitter lines */
    apb_uart1_control(APB_UART_CONTROL_TX, 1);			// Transmit
    apb_uart1_control(APB_UART_CONTROL_RX, 1);			// Received

    apb_uart1_send("\r\nInput 'G' to receive a message", 34);
    wait_complete();

    // Input character "G"
    // Print "Hello World!"
    while (1)
    {
    	// Receive 'G'
    	apb_uart1_receive(&cmd, 1);           /* Get a character 'G' from APB UART1 */
        wait_complete();
        if (cmd == 'G')                      /* 'G', send greeting  */
        {
        	apb_uart1_send("\r\nHello World!", 14);
        	wait_complete();
        }
    }

    return 0;
}
